@@ -28,6 +28,7 @@
 
 
 #define STONE_SIZE 32
+#define SCROLL_SPEED 26
 
 
 typedef struct _GStonesPlugin       GStonesPlugin;
@@ -83,7 +84,7 @@ typedef enum
   GSTONES_TYPE_INT,
   GSTONES_TYPE_BOOL,
   GSTONES_TYPE_STRING,
-  GSTONES_TYPE_TIME,
+  GSTONES_TYPE_TIME
 } GStonesOptionType;
 
 
@@ -211,8 +212,8 @@ struct _GStonesCave
 
   /* Some static information about this cave.  */
   GStonesGame      *game;
-  guint             width;
-  guint             height;
+  gint              width;
+  gint              height;
   gboolean          is_intermission;
   guint             diamond_score;
   guint             extra_diamond_score;
@@ -242,6 +243,9 @@ struct _GStonesCave
 
   GStonesCaveEntry  entry[CAVE_MAX_WIDTH+2][CAVE_MAX_HEIGHT+2];
 
+  /* Animated curtain */
+  GStonesObject *animated_curtain;
+
   /* This hash table holds an object context for every object that is used in
      this cave.  */
   GHashTable       *contexts;
