@@ -48,47 +48,44 @@
 #include "yahtzee.h"
 #include "gyahtzee.h"
 
-#include "pix/gnome-dice-1.xpm"
-#include "pix/gnome-dice-2.xpm"
-#include "pix/gnome-dice-3.xpm"
-#include "pix/gnome-dice-4.xpm"
-#include "pix/gnome-dice-5.xpm"
-#include "pix/gnome-dice-6.xpm"
-#include "pix/gnome-dice-none.xpm"
-
 /* Define a sensible alternative to ngettext if we don't have it. Note that
  * this is only sensible in the context of gtali. */
 #ifndef HAVE_NGETTEXT
 #define ngettext(one,lots,n) gettext(lots)
 #endif
 
+#define DELAY_MS 600
+
 int GyahtzeeAbort = 0;  /* Abort program without playing game */
 
 static char *appID="gtali";
 static char *appName=N_("GNOME Tali");
 static guint lastHighScore = 0;
-GtkWidget *ScoreList; 
+static guint last_timeout = 0;
+static gboolean ready_to_advance_player;
 
 #ifdef GNOMEPIXMAPDIR
-#define PP GNOMEPIXMAPDIR "/" 
+#define PP GNOMEPIXMAPDIR "/gtali/" 
 #else
 #define PP ""
 #endif
 
-static char *dicefiles[NUMBER_OF_PIXMAPS] = { PP "gnome-dice-1.xpm",
-                                              PP "gnome-dice-2.xpm",
-                                              PP "gnome-dice-3.xpm",
-                                              PP "gnome-dice-4.xpm",
-                                              PP "gnome-dice-5.xpm",
-                                              PP "gnome-dice-6.xpm",
-                                              PP "gnome-dice-none.xpm" };
+static char *dicefiles[NUMBER_OF_PIXMAPS] = { PP "gnome-dice-1.svg",
+                                              PP "gnome-dice-2.svg",
+                                              PP "gnome-dice-3.svg",
+                                              PP "gnome-dice-4.svg",
+                                              PP "gnome-dice-5.svg",
+                                              PP "gnome-dice-6.svg",
+                                              PP "gnome-dice-none.svg" };
 
 static GtkWidget *dicePixmaps[NUMBER_OF_DICE][NUMBER_OF_PIXMAPS];
 
 GtkWidget *window;
+GtkWidget *ScoreList;
 static GtkWidget *appbar, *diceBox[NUMBER_OF_DICE];
 static GtkWidget *diceTable;
 static GtkWidget *rollLabel;
+static GtkWidget *mbutton;
 
 static gint gnome_modify_dice (GtkWidget *widget, GdkEventButton *event,
                                gpointer data);
@@ -96,15 +93,9 @@ static gint gnome_roll_dice (GtkWidget *widget, GdkEvent *event,
                              gpointer data);
 void update_score_state (void);
 static void undo_set_sensitive (gboolean state);
+static void roll_set_sensitive (gboolean state);
 static void UpdateRollLabel (void);
 
-void
-YahtzeeIdle (void)
-{
-        while (gtk_events_pending ())
-                gtk_main_iteration ();
-}
-
 
 static void
 CheerWinner (void)
@@ -134,18 +125,53 @@ CheerWinner (void)
         undo_set_sensitive (FALSE);
 }
 
+static gboolean
+do_computer_turns (void)
+{
+        if (!players[CurrentPlayer].comp) {
+                last_timeout = 0;
+                return FALSE;
+        }
+
+        if (ready_to_advance_player){
+                NextPlayer ();
+                return TRUE;
+        }
+
+        if (players[CurrentPlayer].finished){
+                NextPlayer();
+                return TRUE;
+        }
+
+        ComputerRolling(CurrentPlayer);
+        if ( NoDiceSelected() || (NumberOfRolls>=NUM_ROLLS) ) {
+                ComputerScoring(CurrentPlayer);
+                ready_to_advance_player = TRUE;
+        } else {
+                RollSelectedDice();
+                UpdateRollLabel();
+        }
+
+        if (!DoDelay)
+                do_computer_turns();
+
+        return TRUE;
+}
+
 void 
 NextPlayer (void)
 {
         if (GameIsOver ()) {
+                if (last_timeout){
+                        g_source_remove (last_timeout);
+                        last_timeout = 0;
+                }
                 CheerWinner ();
                 return;
         }
 
-        undo_set_sensitive (TRUE);
-        
         NumberOfRolls = 0;
-
+        ready_to_advance_player = FALSE;
 	ShowoffPlayer (ScoreList,CurrentPlayer,0);
 
         /* Find the next player with rolls left */
@@ -167,11 +193,18 @@ NextPlayer (void)
 
         SelectAllDice ();
         RollSelectedDice ();
+	
+	undo_set_sensitive (!players[CurrentPlayer].comp);
 
         if (players[CurrentPlayer].comp) {
-                ComputerTurn (CurrentPlayer);
-                NextPlayer ();
-                return;
+                if (DoDelay){
+                        if (!last_timeout)
+                                last_timeout = g_timeout_add (DELAY_MS,
+                                                              (GSourceFunc)do_computer_turns,
+                                                              NULL);
+                } else {
+                       do_computer_turns();
+                }
         }
         UpdateRollLabel ();
 }
@@ -260,12 +293,6 @@ GyahtzeeNewGame (void)
                 ShowoffPlayer (ScoreList,i,0);
 	ShowoffPlayer (ScoreList,0,1);
 
-        /* All players are computers, start game immediately */
-        if (players[CurrentPlayer].comp) {
-                CurrentPlayer = NumberOfComputers - 1;
-                NextPlayer ();
-        }
-
         undo_set_sensitive (FALSE);
 }
 
@@ -287,6 +314,8 @@ UpdateRollLabel (void)
         
         g_string_printf(str, "<b>%s %d/3</b>", _("Roll"), NumberOfRolls);
         gtk_label_set_label(GTK_LABEL(rollLabel), str->str);
+	
+        roll_set_sensitive ((NumberOfRolls < 3) && !players[CurrentPlayer].comp);
 }
 
 static void
@@ -335,6 +364,9 @@ gnome_modify_dice (GtkWidget *widget, GdkEventButton *event, gpointer data)
 	if (players[CurrentPlayer].finished)
                 return TRUE;
 
+	if (players[CurrentPlayer].comp)
+		return TRUE;
+
         if (NumberOfRolls >= NUM_ROLLS) {
                 say(_("You're only allowed three rolls! Select a score box."));
                 return TRUE;
@@ -519,20 +551,16 @@ static void undo_set_sensitive (gboolean state)
         gtk_widget_set_sensitive (gamemenu[2].widget, state);
 }
 
+static void roll_set_sensitive (gboolean state)
+{
+	gtk_widget_set_sensitive (mbutton, state);
+}
+
 static void
 LoadDicePixmaps(void)
 {
         GtkWidget *tmp;
 	GdkPixbuf *pixbuf;
-	const char **xpm_data[] = {
-		gnome_dice_1_xpm,
-		gnome_dice_2_xpm,
-		gnome_dice_3_xpm,
-		gnome_dice_4_xpm,
-		gnome_dice_5_xpm,
-		gnome_dice_6_xpm,
-		gnome_dice_none_xpm
-	};
 	int i, j;
 
 	for (i=0; i < NUMBER_OF_PIXMAPS; i++) {
@@ -542,13 +570,7 @@ LoadDicePixmaps(void)
 		if (g_file_test (dicefiles[i], G_FILE_TEST_EXISTS))
 			tmp = gtk_image_new_from_file (dicefiles[i]);
 
-		if (tmp != NULL) {
-			dicePixmaps[0][i] = tmp;
-		} else { /* Load default dice */
-			pixbuf = gdk_pixbuf_new_from_xpm_data (xpm_data[i]);
-			dicePixmaps[0][i] = gtk_image_new_from_pixbuf (pixbuf);
-			g_object_unref (pixbuf);
-		}
+                dicePixmaps[0][i] = tmp;
 
 		for (j=0; j < NUMBER_OF_DICE; j++) {
 			pixbuf = gtk_image_get_pixbuf 
@@ -581,7 +603,6 @@ static void
 GyahtzeeCreateMainWindow(void)
 {
         GtkWidget *all_boxes;
-	GtkWidget *mbutton;
 	GtkWidget *tmp;
 	int i, j;
 
@@ -727,7 +748,6 @@ main (int argc, char *argv[])
         DisplayComputerThoughts = gconf_client_get_bool (client,
                                                          "/apps/gtali/DisplayComputerThoughts",
                                                          NULL);
-        
         /* Read in new player names */
         name_list = gconf_client_get_list (client, "/apps/gtali/PlayerNames",
                                            GCONF_VALUE_STRING, &err);
