@@ -90,14 +90,6 @@ char *FieldLabels[NUM_FIELDS+EXTRA_FIELDS] =
 	N_("Bonus if >62"),
 };
 
-void
-YahtzeeDelay(void)
-{
-        YahtzeeIdle();
-        if (DoDelay)
-                sleep(COMPUTER_DELAY);		/* let person read it */
-}
-
 int 
 NoDiceSelected(void)
 {
@@ -436,7 +428,8 @@ play_score(int player, int field)
 
         case 8:
                 if ( (i = find_n_of_a_kind(3, 0)) ) {
-                        if (find_n_of_a_kind(2, i))
+                        /* We treat a yahtzee as a full house. */
+                        if (find_n_of_a_kind(2, i) || find_n_of_a_kind(5,0))
                                 players[player].score[field] = 25;
                 }
                 break;
